/**
 * Copyright (c) 2014, 2015 Christian W. Damus and others.
 * 
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 * 
 * Contributors:
 *   Christian W. Damus - Initial API and implementation
 */
package org.eclipse.papyrus.uml.profile.elementtypesconfigurations.generator;

import com.google.common.base.Objects;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import javax.inject.Inject;
import javax.inject.Singleton;
import org.eclipse.core.resources.IProject;
import org.eclipse.core.resources.IWorkspace;
import org.eclipse.core.resources.IWorkspaceRoot;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EObject;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.papyrus.infra.elementtypesconfigurations.ElementTypeConfiguration;
import org.eclipse.papyrus.infra.elementtypesconfigurations.ElementtypesconfigurationsFactory;
import org.eclipse.papyrus.infra.elementtypesconfigurations.IconEntry;
import org.eclipse.papyrus.infra.elementtypesconfigurations.SpecializationTypeConfiguration;
import org.eclipse.papyrus.uml.profile.elementtypesconfigurations.generator.Identifiers;
import org.eclipse.papyrus.uml.profile.elementtypesconfigurations.generator.ImpliedExtension;
import org.eclipse.papyrus.uml.profile.elementtypesconfigurations.generator.UMLElementTypes;
import org.eclipse.papyrus.uml.tools.elementtypesconfigurations.stereotypeapplicationmatcherconfiguration.StereotypeApplicationMatcherConfiguration;
import org.eclipse.papyrus.uml.tools.elementtypesconfigurations.stereotypeapplicationmatcherconfiguration.StereotypeApplicationMatcherConfigurationFactory;
import org.eclipse.uml2.uml.Image;
import org.eclipse.uml2.uml.Stereotype;
import org.eclipse.xtext.xbase.lib.CollectionLiterals;
import org.eclipse.xtext.xbase.lib.Extension;
import org.eclipse.xtext.xbase.lib.Functions.Function1;
import org.eclipse.xtext.xbase.lib.IterableExtensions;
import org.eclipse.xtext.xbase.lib.ObjectExtensions;
import org.eclipse.xtext.xbase.lib.Procedures.Procedure1;
import org.eclipse.xtext.xbase.lib.StringExtensions;

/**
 * Transformation rule for generating a {@link SpecializationTypeConfiguration} from a UML metaclass {@link Extension}.
 */
@Singleton
@SuppressWarnings("all")
public class ElementTypeRule {
  @Extension
  private static ElementtypesconfigurationsFactory elementtypesconfigurationsFactory = ElementtypesconfigurationsFactory.eINSTANCE;
  
  @Extension
  private static StereotypeApplicationMatcherConfigurationFactory stereotypeApplicationMatcherConfigurationFactory = StereotypeApplicationMatcherConfigurationFactory.eINSTANCE;
  
  @Inject
  @Extension
  private UMLElementTypes _uMLElementTypes;
  
  @Inject
  @Extension
  private Identifiers _identifiers;
  
  public SpecializationTypeConfiguration toElementType(final ImpliedExtension umlExtension, final ElementTypeConfiguration supertype) {
    final ArrayList<?> _cacheKey = CollectionLiterals.newArrayList(umlExtension, supertype);
    final SpecializationTypeConfiguration _result;
    synchronized (_createCache_toElementType) {
      if (_createCache_toElementType.containsKey(_cacheKey)) {
        return _createCache_toElementType.get(_cacheKey);
      }
      SpecializationTypeConfiguration _createSpecializationTypeConfiguration = ElementTypeRule.elementtypesconfigurationsFactory.createSpecializationTypeConfiguration();
      _result = _createSpecializationTypeConfiguration;
      _createCache_toElementType.put(_cacheKey, _result);
    }
    _init_toElementType(_result, umlExtension, supertype);
    return _result;
  }
  
  private final HashMap<ArrayList<?>, SpecializationTypeConfiguration> _createCache_toElementType = CollectionLiterals.newHashMap();
  
  private void _init_toElementType(final SpecializationTypeConfiguration it, final ImpliedExtension umlExtension, final ElementTypeConfiguration supertype) {
    String _elementTypeID = this._identifiers.toElementTypeID(umlExtension, supertype);
    it.setIdentifier(_elementTypeID);
    boolean _hasSemanticSupertype = this._uMLElementTypes.hasSemanticSupertype(supertype);
    if (_hasSemanticSupertype) {
      EList<String> _specializedTypesID = it.getSpecializedTypesID();
      org.eclipse.uml2.uml.Class _metaclass = umlExtension.getMetaclass();
      ElementTypeConfiguration _elementTypeConfiguration = this._uMLElementTypes.getElementTypeConfiguration(_metaclass);
      String _elementTypeID_1 = this._identifiers.toElementTypeID(umlExtension, _elementTypeConfiguration);
      _specializedTypesID.add(_elementTypeID_1);
    }
    EList<String> _specializedTypesID_1 = it.getSpecializedTypesID();
    String _identifier = supertype.getIdentifier();
    _specializedTypesID_1.add(_identifier);
    String _hint = supertype.getHint();
    it.setHint(_hint);
    String _elementTypeName = this._identifiers.toElementTypeName(umlExtension, supertype);
    it.setName(_elementTypeName);
    Stereotype _stereotype = umlExtension.getStereotype();
    IconEntry icon = this.getIconEntry(_stereotype);
    IconEntry _xifexpression = null;
    boolean _notEquals = (!Objects.equal(icon, null));
    if (_notEquals) {
      _xifexpression = icon;
    } else {
      org.eclipse.uml2.uml.Class _metaclass_1 = umlExtension.getMetaclass();
      _xifexpression = this._uMLElementTypes.getIconEntry(_metaclass_1);
    }
    it.setIconEntry(_xifexpression);
    boolean _hasSemanticSupertype_1 = this._uMLElementTypes.hasSemanticSupertype(supertype);
    boolean _not = (!_hasSemanticSupertype_1);
    if (_not) {
      StereotypeApplicationMatcherConfiguration _matcherConfiguration = this.toMatcherConfiguration(umlExtension, supertype);
      it.setMatcherConfiguration(_matcherConfiguration);
    }
  }
  
  private StereotypeApplicationMatcherConfiguration toMatcherConfiguration(final ImpliedExtension umlExtension, final ElementTypeConfiguration supertype) {
    final ArrayList<?> _cacheKey = CollectionLiterals.newArrayList(umlExtension, supertype);
    final StereotypeApplicationMatcherConfiguration _result;
    synchronized (_createCache_toMatcherConfiguration) {
      if (_createCache_toMatcherConfiguration.containsKey(_cacheKey)) {
        return _createCache_toMatcherConfiguration.get(_cacheKey);
      }
      StereotypeApplicationMatcherConfiguration _createStereotypeApplicationMatcherConfiguration = ElementTypeRule.stereotypeApplicationMatcherConfigurationFactory.createStereotypeApplicationMatcherConfiguration();
      _result = _createStereotypeApplicationMatcherConfiguration;
      _createCache_toMatcherConfiguration.put(_cacheKey, _result);
    }
    _init_toMatcherConfiguration(_result, umlExtension, supertype);
    return _result;
  }
  
  private final HashMap<ArrayList<?>, StereotypeApplicationMatcherConfiguration> _createCache_toMatcherConfiguration = CollectionLiterals.newHashMap();
  
  private void _init_toMatcherConfiguration(final StereotypeApplicationMatcherConfiguration it, final ImpliedExtension umlExtension, final ElementTypeConfiguration supertype) {
    EList<String> _stereotypesQualifiedNames = it.getStereotypesQualifiedNames();
    Stereotype _stereotype = umlExtension.getStereotype();
    String _qualifiedName = _stereotype.getQualifiedName();
    _stereotypesQualifiedNames.add(_qualifiedName);
  }
  
  private IconEntry getIconEntry(final Stereotype stereotype) {
    IconEntry _xblockexpression = null;
    {
      EList<Image> _icons = stereotype.getIcons();
      final Function1<Image, Boolean> _function = new Function1<Image, Boolean>() {
        public Boolean apply(final Image it) {
          String _location = it.getLocation();
          boolean _isNullOrEmpty = StringExtensions.isNullOrEmpty(_location);
          return Boolean.valueOf((!_isNullOrEmpty));
        }
      };
      final Image image = IterableExtensions.<Image>findFirst(_icons, _function);
      IconEntry _xifexpression = null;
      boolean _notEquals = (!Objects.equal(image, null));
      if (_notEquals) {
        IconEntry _xblockexpression_1 = null;
        {
          String _location = image.getLocation();
          final URI uri = URI.createURI(_location, true);
          IconEntry _xifexpression_1 = null;
          boolean _notEquals_1 = (!Objects.equal(uri, null));
          if (_notEquals_1) {
            IconEntry _createIconEntry = ElementTypeRule.elementtypesconfigurationsFactory.createIconEntry();
            final Procedure1<IconEntry> _function_1 = new Procedure1<IconEntry>() {
              public void apply(final IconEntry it) {
                boolean _isPlatform = uri.isPlatform();
                if (_isPlatform) {
                  String _segment = uri.segment(1);
                  it.setBundleId(_segment);
                  List<String> _segmentsList = uri.segmentsList();
                  Iterable<String> _drop = IterableExtensions.<String>drop(_segmentsList, 2);
                  final Function1<String, CharSequence> _function = new Function1<String, CharSequence>() {
                    public CharSequence apply(final String it) {
                      return URI.decode(it);
                    }
                  };
                  String _join = IterableExtensions.<String>join(_drop, "/", _function);
                  String _plus = ("/" + _join);
                  it.setIconPath(_plus);
                } else {
                  boolean _isRelative = uri.isRelative();
                  if (_isRelative) {
                    IProject _containingProject = ElementTypeRule.this.containingProject(stereotype);
                    String _name = _containingProject.getName();
                    it.setBundleId(_name);
                    String _string = uri.toString();
                    String _decode = URI.decode(_string);
                    String _plus_1 = ("/" + _decode);
                    it.setIconPath(_plus_1);
                  } else {
                    String _string_1 = uri.toString();
                    it.setIconPath(_string_1);
                  }
                }
              }
            };
            _xifexpression_1 = ObjectExtensions.<IconEntry>operator_doubleArrow(_createIconEntry, _function_1);
          }
          _xblockexpression_1 = _xifexpression_1;
        }
        _xifexpression = _xblockexpression_1;
      }
      _xblockexpression = _xifexpression;
    }
    return _xblockexpression;
  }
  
  private IProject containingProject(final EObject object) {
    IWorkspace _workspace = ResourcesPlugin.getWorkspace();
    IWorkspaceRoot _root = _workspace.getRoot();
    Resource _eResource = object.eResource();
    URI _uRI = _eResource.getURI();
    String _segment = _uRI.segment(2);
    return _root.getProject(_segment);
  }
}
